#include "libc.h"
#include "sim_types.h"
#include "sim_misc.h"

static const char rcsid[] =
"$Id: misc.c,v 1.4 1998/06/06 09:25:49 schwartz Exp schwartz $";

/* fatal ---------------------------------------------- print message and die */
void fatal(const char *msg)
{
        (void)fputs(msg, stderr);
        (void)fputc('\n', stderr);
	exit(1);
}

/* fatalf --------------------------------- format message, print it, and die */
void fatalf(const char *msg, const char *val)
{
	(void)fprintf(stderr, msg, val);
	(void)fputc('\n', stderr);
	exit(1);
}
	
/* ckopen -------------------------------------- open file; check for success */
FILE *ckopen(const char *name, const char *mode)
{
	FILE *fp;

	if ((fp = fopen(name, mode)) == NULL)
		fatalf("Cannot open %s.", name);
	return fp;
}

/* ckalloc -------------------------------- allocate space; check for success */
void *ckalloc(size_t amount)
{
	void *p;

	if ((p = malloc(amount)) == NULL)
		fatal("Ran out of memory.");
	memset(p, 0, amount);
	return p;
}

/* strsame --------------------------- tell whether two strings are identical */
bool same_string(const char *s, const char *t)
{
	return (strcmp(s, t) == 0);
}

/* strsave -------------------------- save string s somewhere; return address */
char *copy_string(const char *s)
{
	char *p;

	p = ckalloc(strlen(s)+1);	/* +1 to hold '\0' */
	return strcpy(p, s);
}

long ckftell(FILE *f)
{
	long r = ftell(f);
	if (r < 0)
		fatalf("bad ftell: %s", strerror(errno));
	return r;
}

int ckfseek(FILE *f, long i, int m)
{
	int r = fseek(f, i, m);
	if (r < 0)
		fatalf("bad fseek: %s", strerror(errno));
	return r;
}
