#include "libc.h"
#include "sim_types.h"
#include "sim_args.h"
#include "sim_seq.h"
#include "sim_dna.h"
#include "sim_misc.h"
#include "sim_prnt.h"

static const char rcsid[] =
"$Id: prnt.c,v 1.3 1998/06/06 09:25:49 schwartz Exp schwartz $";

/* XXX */
static int offset1;
static int offset2;

/* print_align_header  -------------  print the top part of an alignment file */
void print_align_header(SEQ *seq1, SEQ *seq2, argv_scores_t *ds)
{
	int f, t, F, T;

	(void)printf("#:lav\n\nd {\n  \"");
	ck_argc("print_align_header");
	fprintf_argv(stdout);
	if (ds->M) {
		(void)printf("\n   M = %d, I = %d, V = %d", ds->M, ds->I, ds->V);
		if (ds->O) (void)printf(", O = %d, E = %d", ds->O, ds->E);
	} else
		(void)printf("\n   Blosum62 scores");
	if (get_argval('f', &f)) {
		if (!get_argval('t',&t) || !get_argval('F',&F) ||
		    !get_argval('T',&T))
			fatal("Inconsistent use of `f`, `t`, `F', `T' args.");
		(void)printf("\"\n}\ns {\n  \"%s\" %d %d\n  \"%s\" %d %d\n}\n",
			SEQ_NAME(seq1), f, t, SEQ_NAME(seq2), F, T);
		offset1 = SEQ_FROM(seq1) - f;
		offset2 = SEQ_FROM(seq2) - F;
		
	} else {
		(void)printf("\"\n}\ns {\n  \"%s\" %d %d\n  \"%s\" %d %d\n}\n",
			SEQ_NAME(seq1), SEQ_FROM(seq1), SEQ_TO(seq1), 
			SEQ_NAME(seq2), SEQ_FROM(seq2), SEQ_TO(seq2)
		);
		offset1 = offset2 = 0;
	}
	(void)printf("h {\n   \"%s\"\n   \"%s\"\n}\n", SEQ_HEAD(seq1), SEQ_HEAD(seq2));
}

/* print_block  ---------------------------------- print a gap-free alignment */
void print_block(int score, int beg1, int end1, int beg2, int end2, int f)
{
	beg1 += offset1;
	end1 += offset1;
	beg2 += offset2;
	end2 += offset2;

	(void)printf("a {\n  s %d\n  b %d %d\n  e %d %d\n  f %d\n",
		score, beg1, beg2, end1, end2, f);
	(void)printf("  l %d %d %d %d\n}\n", beg1, beg2, end1, end2);
}

/* print_align  ----------------------------------- print a general alignment */
void print_align(int score, uchar *seq1, uchar *seq2, int beg1, int end1, int beg2, int end2,int *S)
{
	int M, N, i, j, op, start_i, start_j, match, run, pct;
	uchar *P, *p, *q;
	
	beg1 += offset1;
  	end1 += offset1;
	beg2 += offset2;
	end2 += offset2;

	M = end1 - beg1 + 1;
	N = end2 - beg2 + 1;
	(void)printf("a {\n  s %d\n  b %d %d\n  e %d %d\n",
		score, beg1, beg2, end1, end2);
	for (i = j = 0; i < M || j < N; ) {
		start_i = i;
		start_j = j;
		match = 0;
		P= p = seq1 + beg1 + i - 1;
		q = seq2 + beg2 + j - 1;
		while (i < M && j < N && *S == 0) {
			if (*p++ == *q++)
				++match;
			++i;
			++j;
			++S;
		}
		run = p - P;
		pct = (run > 0) ? ((100*match + run/2)/run) : 0; /* round */
		(void)printf("  l %d %d %d %d %d\n",
			beg1+start_i, beg2+start_j, beg1+i-1, beg2+j-1, pct);
		if (i < M || j < N)
			if ((op = *S++) > 0)
				j += op;
			else
				i -= op;
    	}
	(void)printf("}\n");
}
