#include "libc.h"
#include "sim_types.h"
#include "sim_seq.h"
#include "sim_misc.h"
#include "sim_args.h"
#include "sim_karlin.h"
#include "sim_dna.h"

static const char rcsid[] =
"$Id: dna.c,v 1.4 1998/06/06 09:25:49 schwartz Exp schwartz $";

static const argv_scores_t EIMOSV = { 2, -10, 10, 60, 0.95, -10 };

/* DNA characters */
const uchar dchars[] = "ABCDGHKMNRSTVWXY";

/* DNA_scores -----------------------------------  set scoring matrix for DNA */
void DNA_scores(argv_scores_t *ds, ss_t ss)
{
	int i, j, bad;

	ck_argc("DNA_scores");
	argv_scores(ds, &EIMOSV);

	for (i = 0; i < NACHARS; ++i)
		for (j = 0; j < NACHARS; ++j)
			ss[i][j] = ds->V;

	bad = -100*ds->M;
	for (i = 0; i < NACHARS; ++i)
		ss['X'][i] = ss[i]['X'] = bad;

	ss['A']['A'] = ss['C']['C'] = ss['G']['G'] = ss['T']['T'] = ds->M;
	ss['A']['G'] = ss['G']['A'] = ss['C']['T'] = ss['T']['C'] = ds->I;
}

/* DNA_thresh ----------- determine cutoff score according to Karlin-Altschul */
int DNA_thresh(SEQ *seq1, SEQ *seq2, argv_scores_t *ds)
{
        double _p[1000], *p, pMatch, pTransit, pTransver, N,
               param_K, param_lambda, x;
        int A1, A2, C1, C2, G1, G2, T1, T2;
        int c, i, len1, len2;
	int Match = ds->M, Transver = ds->V, Transit = ds->I;

	len1 = SEQ_LEN(seq1);
	len2 = SEQ_LEN(seq2);
        A1 = A2 = C1 = C2 = G1 = G2 = T1 = T2 = 0;

        for (i = 0; i < len1; ++i)
                if ((c = SEQ_AT(seq1,i)) == 'A')
                        ++A1;
                else if (c == 'C')
                        ++C1;
                else if (c == 'G')
                        ++G1;
                else if (c == 'T')
                        ++T1;
	len1 = A1 + C1 + G1 + T1;
        for (i = 0; i < len2; ++i)
                if ((c = SEQ_AT(seq2,i)) == 'A')
                        ++A2;
                else if (c == 'C')
                        ++C2;
                else if (c == 'G')
                        ++G2;
                else if (c == 'T')
                        ++T2;
	len2 = A2 + C2 + G2 + T2;

        pMatch = ((double)A1)*((double)A2)
               + ((double)C1)*((double)C2)
               + ((double)G1)*((double)G2)
               + ((double)T1)*((double)T2);
        pTransit = ((double)A1)*((double)G2)
               + ((double)C1)*((double)T2)
               + ((double)G1)*((double)A2)
               + ((double)T1)*((double)C2);
	if (seq1 == seq2) {
	    N = ((double)len1-1)*((double)len1)/2.0;	
	    pMatch = (pMatch-(double) len1)/2.0;
	    pTransit /= 2.0;
	} else {
	    N = ((double)len1)*((double)len2);	    
	}
        pTransver = N - pMatch - pTransit;

        for (i = 0; i <= -Transver+Match; ++i)
                _p[i] = 0.0;
        p = _p - Transver;
        p[Transver] = pTransver/N;
        p[Transit] += (pTransit/N);
        p[Match] += (pMatch/N);

        if (karlin(Transver, Match, _p, &param_lambda, &param_K) == 0)
                fatal("compution of significance levels failed");

        x = log(ds->S)/(-param_K*N);
        return (int)(2*log(sqrt(x))/(-param_lambda));
}

int is_dchar(int ch)
{
	return !!strchr((const char*)dchars, ch);
}

